#ifndef GIBBS_ITERATION_H_
#define GIBBS_ITERATION_H_

#include "data.h"
#include "model.h"


class gibbs_step {

public:
	string SMC_type;
	datanew	DATA1;
	int N;
	double* w;	double** X; 	int** A; double** s_np1;
    float ** w_mat; // matrix of weights; used in backward sampling
    int** ywsum_range;
    double** swsum_range;

	int* itemp_N1; 	int index;

	model2	MODE1;
	proposal PROP1;
	double* part_n;
	double* struct_n;
	double* part_np1;
	double* struct_np1;
	double likelihood;

	int* bk;
	int* bkS;
	int* outlier;
	double* z_n;
	int* indices;
	int* sequence_index;
	double* dtemp_T;
	map<int,pair<int,int> > updates;int update_length;
	map<int,pair<int,int> > updates_struct;int update_length_struct;
	map<int,pair<int,int> > poz_update;

public:

	gibbs_step( string SMC_type, int N, datanew data1, model2	mode, proposal prop);
	void set_u_n(double* u_n){for (unsigned int t=0; t<DATA1.T;t++){      part_n[t]=u_n[t];}}
	void set_s_n(double* s_n){for (unsigned int t=0; t<DATA1.T;t++){      struct_n[t]=s_n[t];}}
	void set_outlier(int* outlier){for (unsigned int t=0; t<DATA1.T;t++){this->outlier[t]=outlier[t];}}
	void set_model(model2 mode){this-> MODE1=mode;}

	void SMC(gsl_rng* rgen);
	void SMC_param(gsl_rng* rgen, data_segments SEGMENTS);

	double* get_u_np1(){	return part_np1;	}
	double* get_s_np1(){	 return struct_np1;	}
	double* get_u_n(){	return part_n;	}
	double* get_s_n(){	return struct_n;	}
	int* get_outlier(){	return outlier;	}

//	NO STEPTS INFERENCE
//	void Csmc_struct_part_level (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int* indices, int N, double* u_n, double* s_n,	double* particle,double* struct_np1);
//	void Csmc_struct_part (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,int* indices, int N, double* u_n, double* s_n,	double* particle);
//	void Csmc_struct_struct (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,int* indices, int N, double* u_n, double* s_n,	double* particle);

	void partlevel_update_segment (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA, int N,  int a,int b,int aS, int bS);

	void CSMC_level (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int N);
	void CSMC_part (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int N);
    void CSMC_part_blocks (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int N);
	void CSMC_struct (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA, int N);
/// simulate counts for given expression profile and parameters ///
    void Simulate_counts(gsl_rng * rgen, model2 MODE, datanew *DATA);

	void CSMC_SEGM_part
	(gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int N, data_segments SEGM);
	void CSMC_SEGM_struct
	(gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int N, data_segments SEGM);
	void CSMC_SEGM_level
	(gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA,	int N, data_segments SEGM);


	void filtering (gsl_rng * rgen, model2 MODE, proposal PROP, datanew DATA, int N);

	virtual ~gibbs_step(){
		//		cout<<"destructor gibbs"<<endl;
		for (int i=0; i<N;i++){	delete [] X[i]; delete[] A[i]; delete[] s_np1[i]; delete[] w_mat[i];}
		delete[] X; delete [] A; delete [] w; delete[]w_mat;
		delete [] z_n;  delete [] s_np1; delete []itemp_N1; delete [] dtemp_T;
		delete [] bk; delete [] bkS;
//		delete[] part_n; 	delete[]struct_n;
//		delete[] part_np1;	delete[] struct_np1;
		delete[] indices; 	delete[] sequence_index; delete[]outlier;
	}






};

class gibbs_step_multipledata:public gibbs_step {

	datanew DATA2;
	model2 MODE2;
	proposal PROP2;

	double* part2_n;
	double* part2_np1;
public:
	gibbs_step_multipledata( string SMC_type, int N, datanew DATA1, model2	MODE1, proposal PROP1, datanew DATA2, model2	MODE2,proposal PROP2);
	void set_model1(model2 mode){this-> MODE1=mode;}
	void set_model2(model2 mode){this-> MODE2=mode;}
	void set_u2_n(double* u_n){for (unsigned int t=0; t<DATA1.T;t++){part2_n[t]=u_n[t];}}
	double* get_u2_np1(){	return part2_np1;	}
	double* get_u2_n(){	return part2_n;	}
	void SMC_MULTIPLE(gsl_rng* rgen);
};


#endif /* GIBBS_ITERATION_H_ */
