
#include "model.h"


/// model
model::model( double xalfa, double xbeta, double xzeta, double xphi, int xnpars){
	alpha=xalfa;
	beta=xbeta;
	zeta=xzeta;
	phi=xphi;
	npars=xnpars;
}

model2::model2( double xalfa, double xbeta, double xbeta_0,
		double xgamma_u, double xgamma_d, double xeta, double xlambda_u, double xlambda_d,
		double xc, double kappa, double theta,double xphi,double xalpha_struct, double xstruct_shape,
		double xepsilon, double xepsilon2, double xephi, double xymax, double xemean,
		int xnpars):model( xalfa, xbeta, xc, xphi, xnpars){

	gamma_d=xgamma_d;
	gamma_u=xgamma_u;
	lambda_u=xlambda_u;
	lambda_d=xlambda_d;
	beta_0=xbeta_0;
	eta=xeta;
	epsilon=xepsilon; epsilon2=xepsilon2; ephi=xephi; ymax=xymax; ymean=xemean;
	alpha_struct=xalpha_struct; struct_shape=xstruct_shape;
}

model2::model2( double* param, double ymax, double ymean,double yphi, int npars, vector<float>* nb_table, int max_u, float discretize_step){
	this->param=new double[npars];
	this->npars=npars;
	alpha=param[0];
	beta_0=param[1];
	beta=param[2];
	gamma_u=param[3];
	gamma_d=param[4];
	eta=param[6];
	lambda_u=param[7];
	lambda_d=param[8];
	phi=param[9];
	zeta=param[10];
	epsilon=param[11];
	epsilon2=param[12];
	this->ymax=ymax;
	this->ymean=ymean;
	this->yphi=yphi;

	alpha_struct=param[15];
	struct_shape=param[16];

	kappa=param[17];
	a=param[18];
	this->nb_table=nb_table;

    this->max_u=max_u;
    this->discretize_step=discretize_step;
}


double model2::shift (double ut1){	return gsl_ran_exponential_pdf(ut1, 1/param[10]);}
double model2::shift_0 (double ut1){	return gsl_ran_exponential_pdf(ut1, 1/param[10]);}

double model2::drift_up (double ut1,double ut){	return lambda_u/ut*exp(-lambda_u/ut*(ut1-ut));}
double model2::drift_down (double ut1,double ut){ return lambda_d*ut/pow(ut1,2)*exp(-lambda_d/ut1*(ut-ut1));}


///kernell is a mixture function

double model2::kernel_hidden (double ut1, double ut){
	if(ut<litle){
		if (ut1<litle) { return 1-eta;}
		else {		return	eta*shift_0(ut1);}
	}
	else{
		if (abs(ut-ut1)<litle) { return alpha;}
		else {
			if (ut1 < litle) {return beta_0;}
			if( ut1 > ut) {return	beta*shift(ut1)+gamma_u*drift_up(ut1,ut);}
			if (ut1 < ut)  {return	beta*shift(ut1)+gamma_d*drift_down(ut1,ut) ;}
		}
	}
	cerr<<"Error in kernel_hidden. Tried to compute "<<ut1<<" | "<<ut<<endl; 	exit(2);
}

double model2::sample_kernel (gsl_rng* rgen,double ut){

    float ran=gsl_rng_uniform (rgen);

    if(ut<litle){
		if (ran<eta) { return gsl_ran_exponential(rgen, 1/zeta);}
		else {		return	0;}
	}
	else{
		if (ran<alpha) { return ut;}
		else if (ran<alpha+beta_0) {return 0;}
		else if (ran<alpha+beta_0+beta) {return	gsl_ran_exponential(rgen, 1/zeta);}
		else if (ran<alpha+beta_0+beta+gamma_u) {return	ut*(1+gsl_ran_exponential(rgen,1/lambda_u));}
        else  {return	ut/(1+gsl_ran_exponential(rgen,1/lambda_u));}
		}

	cerr<<"error in kernel hidden"<<endl;
	exit(2);
}
/// kernel for first position
double model2::kernel_hidden_0 (double ut1){
	double v=beta_0/(beta_0+eta);
	if (ut1<litle) { return v;}
	else {
		return (1-v)*shift(ut1);}
}

double model2::shift_struct(double st1){
	return gsl_ran_gamma_pdf(st1, 1.0/struct_shape, struct_shape);}

double model2::kernel_struct_0(double st){
	return shift_struct(st);}

double model2::kernel_struct(double st1,double st){
	if(abs(st-st1)<litle){ return alpha_struct;}
	else {return (1-alpha_struct)*shift_struct(st1);}
}

//////////////EMISSSION DENSITIES

double model2::emission_background (int y, double context){
	if(y>0) {return gsl_ran_negative_binomial_pdf(y,1/(kappa*a+1),1/kappa)/(1-1/pow(a*kappa+1,1/kappa));}
	else return 0;
}

double model2::emission_outlier (int y){
	return (double)1/ymax;//// should be ymax+1
}


double model2::emission (int y, double mean, double context,int max_u,float discretize_step){

    if(mean<0.001) {if(y==0) return 1; else return 0;}
    if (mean>max_u) mean=max_u;
    float lmean=log(mean)*discretize_step+138.1552;//138=20*log(0.001) which is teh lower bound for u
    unsigned int indexmean=(int)floor(lmean);

    if(y>nb_table[indexmean][1]) y=(int)nb_table[indexmean][1];/// if y>ymax set y=ymax

    return ((nb_table[indexmean+1][0]-mean)* nb_table[indexmean][y+2]+(mean-nb_table[indexmean][0])*nb_table[indexmean+1][y+2])/
            (nb_table[indexmean+1][0]-nb_table[indexmean][0]);
}

double model2::emission_proba_struct_duplicated (int y, double mean,double structure, double context, int outlier,int duplicated){

    if(duplicated) return 1;
	if(outlier==0){
		if((mean*structure)<litle){  if(y==0) return 1; else return 0;}
		else {return emission (y, mean*structure,context, max_u, discretize_step);}
		}
	else if(outlier==1){
		return emission_background(y,context);}
	else if (outlier==2){
		return emission_outlier(y);}
	else {
	     cerr<<"error in outlier notation"<<endl;
	     exit(2);
	    }
}


// model parameters are updated

void model2::update_param(double* param){
	alpha=param[0];
	beta_0=param[1];
	beta=param[2];
	gamma_u=param[3];
	gamma_d=param[4];
	eta=param[6];
	lambda_u=param[7];
	lambda_d=param[8];
	phi=param[9];
	zeta=param[10];
	epsilon=param[11];
	epsilon2=param[12];
	ymax=param[13];
    yphi=param[14];
	alpha_struct=param[15];
	struct_shape=param[16];
    kappa=param[17];
	a=param[18];
}

void model2::update_param_struct(double* param){

	alpha_struct=param[15];
	struct_shape=param[16];
}

void model2::update_ymean_ymax(double ymean, double ymax){
	this->ymean=ymean;
	this->ymax=ymax;
}
void model2::update_nb_table(vector<float>* nb_table){
	this->nb_table=nb_table;
}
double* model2::get_param(){
	param[0]=alpha;
	param[1]=beta_0;
	param[2]=beta;
	param[3]=gamma_u;
	param[4]=gamma_d;
    param[5]=1-eta;
	param[6]=eta;
	param[7]=lambda_u;
	param[8]=lambda_d;
	param[9]=phi;
	param[10]=zeta;
	param[11]=epsilon;
	param[12]=epsilon2;
	param[13]=ymax;
    param[14]=0;
	param[15]=alpha_struct;
	param[16]=struct_shape;

    param[17]=kappa;
	param[18]=a;
	return param;
}



// proposal
// there will be 2 main functions in proposal: proba and sample.
// one is to calculate a density and the other to sample form it
// proposal is  a mixture

//proposal PROP1(0,1,DATA1.ywsum[0],eta_p,alpha_p,alpha_p_struct, at, bt, ct, epsilon_p, 20 , 0, window_prop);
//double eta_p=0.1, alpha_p=0.9, at=0.33, bt=0.33 ,ct=1-at-bt , epsilon_p=0.001, alpha_p_struct=0.9;

///***************************************************************************************************************************************************///

proposal::proposal( double xut, double xst, int xywmt,	 bool duplicated,	 double xalpha_p_struct,   double  xzetap, int window){

	eta_p=0.1;
	ut=xut;st=xst;
	this->duplicated=duplicated;
	zetap=xzetap;
	alpha_p_struct=xalpha_p_struct;
	this->window=window;
	MODE=new model2;


	window_range=new int [window];
	ywmt_range=new int[window]; swmt_range=new double[window];
	wmixture=new double[window+3]; wmixture_cum=new double[window+3];

    wmixture[0]=0.97;     wmixture[1]=0.0149;     wmixture[2]=0.0149;
    for (int t=0;t<window; t++)
       window_range[t]=floor(pow(3,t+1));
    for (int t=0;t<window; t++){
       wmixture[t+3]=(1-wmixture[0]-wmixture[1]-wmixture[2])/window;
    }
    wmixture_cum[0]=wmixture[0];
    for (int t=1;t<window+3;t++)
        wmixture_cum[t]=wmixture_cum[t-1]+wmixture[t];
    if(abs(wmixture_cum[window+2]-1)>litle) {cerr<<"Errors in additioning proposal weights!"<<endl;exit(2);}
}

double proposal::sample(gsl_rng* rgen){

	float ran=gsl_rng_uniform (rgen);
    if(!duplicated){
//       cout<<"eta_p: "<<eta_p<<" "<<zetap<<" "<<swmt_range[0] <<" "<< ywmt_range[0]<<" " <<wmixture_cum[0]<<" "<<wmixture_cum[1]<<" " <<wmixture_cum[2]<<" "<<wmixture_cum[3]<<" "<<wmixture_cum[4]<<endl;
        if(ut<litle){
            if(ran<eta_p)
                return gsl_ran_exponential(rgen,zetap);
            else
                return 0;
        }
        else {
            if (ran<=wmixture_cum[0])
                return ut;
            if (ran<=wmixture_cum[1])
                return 0;
            if (ran<=wmixture_cum[2])
                return gsl_ran_exponential(rgen,zetap);

            for (int d=3;d<window+3;d++)
                if(ran<=wmixture_cum[d])
                    return gsl_ran_gamma (rgen,(ywmt_range[d-3]+1),1/(swmt_range[d-3]+1.0/zetap));

            cerr<<"Errors in proposal mixture: "<<ran<<" "<<wmixture_cum[0]<<" "<<wmixture_cum[1]<<" "<<wmixture_cum[2]<<" "<<wmixture_cum[3]<<" "<<wmixture_cum[4]<<" "<<wmixture_cum[5]<<endl;  exit(2);
            cerr<<"Errors in proposal mixture: "<<wmixture_cum[0]<<" "<<wmixture_cum[1]<<endl;  exit(2);
        }
    }
    else
        return gsl_ran_exponential(rgen,zetap);
}

double proposal::proba(double u){

   if(!duplicated){
        if(ut<litle){
            if(u<litle)
                return 1-eta_p;
            else
                return eta_p*gsl_ran_exponential_pdf (u, zetap);
        }
        else {
            if(abs(u-ut)<litle)
                return wmixture[0];
            if(u<litle)
                return wmixture[1];
            float temp=wmixture[2]*gsl_ran_exponential_pdf (u, zetap);
            for (int d=3;d<window+3;d++)
                temp=temp+wmixture[d]*gsl_ran_gamma_pdf(u,(ywmt_range[d-3]+1),1/(swmt_range[d-3]+1.0/zetap));
            return temp;
        }
    }
    else
        return gsl_ran_exponential_pdf (u, zetap);
}

void proposal::proba_components(double* temp, double u){

    for(int d=0;d<window+4;d++) temp[d]=0;

    if(!duplicated&&(ut>=litle)){

        if(abs(u-ut)<litle){
            temp[0]=wmixture[0];
            temp[window+3]=temp[0];
        }

        else if(u<litle){
            temp[1]=wmixture[1];
            temp[window+3]=temp[1];
        }

        else {
            temp[2]=wmixture[2]*gsl_ran_exponential_pdf (u, zetap);

            temp[window+3]=temp[2];
            for (int d=3;d<window+3;d++){
                temp[d]=wmixture[d]*gsl_ran_gamma_pdf(u,(ywmt_range[d-3]+1),1/(swmt_range[d-3]+1.0/zetap));
                temp[window+3]+=temp[d];
            }
        }
    }
}

double proposal::sample_poz0(gsl_rng* rgen){
	double random=gsl_rng_uniform (rgen);
	if(random <= eta_p)
        return gsl_ran_exponential(rgen, zetap);
    return 0;
}

double	proposal::proba_poz0(double u){

	if(u<litle)
        return 1-eta_p;
	else
        return eta_p*gsl_ran_exponential_pdf(u,zetap);
}

double proposal::sample_struct_0(gsl_rng* rgen){return gsl_ran_exponential(rgen, 1);}

double proposal::proba_struct_0(double s){return gsl_ran_exponential_pdf(s, 1);}

double proposal::sample_struct(gsl_rng* rgen){
	double sample;
	double ran=gsl_rng_uniform (rgen);
	if (ran<=alpha_p_struct) {sample=st;}
	// half of samples are independent and half depend on y,s::: hard coded
	else if (ran<=alpha_p_struct+(1-alpha_p_struct)*0.5) { sample=gsl_ran_gamma (rgen,(ywmt+1),1/(window*ut+1));}
	else{ sample=gsl_ran_exponential(rgen, 1);}
	return sample;
}

double	proposal::proba_struct(double s){
	if (indicator(st,s)==1) { return alpha_p_struct;}
	else {	return (1-alpha_p_struct)*(0.5*gsl_ran_gamma_pdf(s,ywmt+1,1/(window*ut+1))+0.5*gsl_ran_exponential_pdf(s,1));}
}

level_estimation::level_estimation(int T, int* counts, model2 mode,double* context, data_segments segm){

	this->T=T;
	this ->counts=counts;

	this ->context=context;
	MODE=mode;
	SEGM=segm;
	bk= new int[T-1]; for (int t=0;t<(T-1);t++){bk[t]=0;}
	bks= new int[T]; for (int t=0;t<(T-1);t++){bks[t]=0;}
	outlier= new int[T]; for (int t=0;t<T;t++){outlier[t]=0;}
	count_bk=new int[MODE.npars]; 	for (int i=0;i<MODE.npars;i++){count_bk[i]=0;}
	count_bkt=new double[5]; 	for (int i=0;i<5;i++){count_bkt[i]=0;}
	param=new double[MODE.npars];
	error_number = new double[3];
}

level_estimation::level_estimation(int T, int* counts, model2 mode,proposal prop, double* context){

	this->T=T;
	this ->counts=counts;

	this ->context=context;
	MODE=mode;
	PROP=prop;
    u= new double[T];     s= new double[T];
	bk= new int[T-1]; for (int t=0;t<(T-1);t++){bk[t]=0;}
	bks= new int[T]; for (int t=0;t<(T-1);t++){bks[t]=0;}
	outlier= new int[T]; for (int t=0;t<T;t++){outlier[t]=0;}
	count_bk=new int[MODE.npars]; 	for (int i=0;i<MODE.npars;i++){count_bk[i]=0;}
	count_bkt=new double[5]; 	for (int i=0;i<5;i++){count_bkt[i]=0;}
	param=new double[MODE.npars];for (int t=0;t<MODE.npars;t++){param[t]=MODE.get_param()[t];}
	error_number = new double[3];
}

level_estimation::~level_estimation(){
	delete [] bk;	delete [] bks; delete [] count_bk; delete [] param;delete [] outlier;
	delete [] error_number; delete [] count_bkt;
}

void level_estimation::update_estimation (double* u, double* s, model2 mode1 ){


	            this->u=u;	this->s=s;
	MODE=mode1;
}

void level_estimation::update_estimation_counts (int* counts){
	this->counts=counts;

}

void level_estimation::update_estimation_context (double* context){
	this->context=context;
}

void level_estimation::sample_particle_break(gsl_rng* rgen ){

	for(int j=0;j<MODE.npars;j++){ count_bk[j]=0;}
	double* weight= new double[3];
	double p,w1,w2;
	int* indice=new int[3];	for (int i=0;i<3;i++){ indice[i]=(2+i);}

	for (int t=0;t<(T-1);t++){
		/* alfa=0, beta_0=1, beta=2, beta=9, gama_u=3, gama_d=4  1- eta= 5, eta=6;*/
		//ONLY In bk the shifts are coded different 2 for shift up and 9 for shift down
		// in the bk count they are joined under the code 2

		if (u[t]<litle){
			if(abs(u[t]-u[t+1])<litle){bk[t]=5;}
			else {bk[t]=6; }

			}

		else {

			if(abs(u[t]-u[t+1])<litle){bk[t]=0;}
			else {
				if (u[t+1]<litle) { bk[t]= 1;}
				else {
					w1= MODE.beta*MODE.shift(u[t+1]);
					p=gsl_rng_uniform (rgen);
					if (u[t+1]>u[t]) {
						w2= MODE.gamma_u*indicator(0, u[t+1],u[t])* MODE.drift_up(u[t+1],u[t]);
						if(p<w1/(w1+w2))bk[t]=2; else bk[t]=3;
					}
					else {
						w2= MODE.gamma_d*indicator(0,u[t],u[t+1])*MODE.drift_down(u[t+1],u[t]);
						if(p<w1/(w1+w2))bk[t]=9; else bk[t]=4;
					}
				}
			}
		}
		switch (bk[t]) {
		case 9:
			count_bk[2]++;
			break;
		default:
			count_bk[bk[t]]++;
			break;
		}
	}

	delete []indice;	delete []weight;
}

void level_estimation::sample_param(gsl_rng * rgen, bool alpha, bool beta_0, bool beta, bool gamma,bool gamma_2, bool lambda,bool zeta,bool precision,bool eta){
	/// Sample TRANSITION parameters

	for (int i=0;i<5;i++){ count_bkt[i]=count_bk[i];	}

	/// apply priors
	count_bkt[0]= count_bkt[0]+100;	count_bkt[1]= count_bkt[1]+1;
	count_bkt[2]= count_bkt[2]+0.5;	count_bkt[3]= count_bkt[3]+2;	count_bkt[4]= count_bkt[4]+3;


	if(alpha&beta_0&beta&gamma){

		gsl_ran_dirichlet (rgen, 5,count_bkt, param);
	}


	else if(!alpha&beta_0&beta&gamma){

		count_bkt[0]=count_bkt[4];
		gsl_ran_dirichlet (rgen, 4,count_bkt, param);
		param[4]=param[0]*(1-MODE.alpha);
		param[1]=param[1]*(1-MODE.alpha);		param[2]=param[2]*(1-MODE.alpha);		param[3]=param[3]*(1-MODE.alpha);

		param[0]=MODE.alpha;
	}

	else if(alpha&	!beta_0&	beta&	!gamma&! gamma_2){

		param[0]=gsl_ran_beta (rgen, count_bkt[0], count_bkt[2]); param[2]=1-param[0];
		param[1]=MODE.beta_0; param[3]=MODE.gamma_u; param[4]=MODE.gamma_d;}


	else if(alpha&	beta_0&		beta&	!gamma & !gamma_2){
		gsl_ran_dirichlet (rgen, 3,count_bkt, param);
		param[3]=MODE.gamma_u;param[4]=MODE.gamma_d;}

	else if(alpha&	beta_0&		beta&	!gamma & gamma_2){
		double gammas=MODE.gamma_u+MODE.gamma_d;

		gsl_ran_dirichlet (rgen, 3,count_bkt, param);
		param[0]=param[0]*(1-gammas); param[1]=param[1]*(1-gammas); param[2]=param[2]*(1-gammas);

		param[3]=gsl_ran_beta(rgen,count_bkt[3], count_bkt[4])*gammas;
		param[4]=gammas-param[3];
	}


	else if(alpha&	!beta_0&		beta&	gamma){
		count_bkt[1]=count_bkt[4];
		gsl_ran_dirichlet (rgen, 4,count_bkt, param);
		param[4]=param[1];param[1]=MODE.beta_0;}


	else if  (!alpha&	!beta_0&	!beta&	!gamma){ 	param[0]=MODE.alpha;param[1]=MODE.beta_0;param[2]=MODE.beta;	param[3]=MODE.gamma_u;param[4]=MODE.gamma_d;}

	if(eta){param[5]=gsl_ran_beta (rgen, (count_bk[5]+100), (count_bk[6]+1));	param[6]=1-param[5];}
	else {	param[6]=MODE.eta; param[5]=1-param[6];}

	/* Sample AMPLITUDE  parameters */

	double diff_u=0, diff_d=0;
	double diff_shift=0, diff_log_shift=0, SS_log_shift=0;
	for (int t=0;t<(T-1);t++){
		if (bk[t]==3) {diff_u= diff_u+(u[t+1]-u[t])/u[t];}
		if (bk[t]==4) {diff_d= diff_d+(u[t]-u[t+1])/u[t+1];}
		if (bk[t]==2||bk[t]==6||bk[t]==9) {

			diff_shift= diff_shift+u[t+1]; diff_log_shift= diff_log_shift+log(u[t+1]);
			SS_log_shift=SS_log_shift+pow((log(u[t+1])-MODE.zeta),2);}
	}
	int shifts=count_bk[2]+count_bk[6];

	if(lambda){

		/* priors for  lambda param = exp(rate= 0.1) */
		param[7]= gsl_ran_gamma (rgen, (count_bk[3]+1),1/(0.1+diff_u) );
		param[8]=  gsl_ran_gamma (rgen, (count_bk[4]+1),1/(0.1+diff_d) );
	}
	else {param[7]= MODE.lambda_u; param[8]=MODE.lambda_d;}

	if(zeta==1){

		double priorscale=1, priorshape=1; //mean and precision of gamma prior
		param[10]=	gsl_ran_gamma(rgen, priorshape+shifts, 1/(1/priorscale+diff_shift));
	}


	else {	param[10]=MODE.zeta;}

}

void level_estimation::sample_outlier(gsl_rng* rgen,bool yes ){
	if(yes){

		int* ind=new int[3];	for (int i=0;i<3;i++){ind[i]=i;}	double* weight= new double[3];

		for (int t=0;t<(T-1);t++){

			weight[0]= (1-MODE.epsilon-MODE.epsilon2)*MODE.emission(counts[t],s[t]*u[t],context[t],MODE.max_u,MODE.discretize_step);
			weight[1]= MODE.epsilon*MODE.emission_background(counts[t],context[t]);
			weight[2]= MODE.epsilon2*MODE.emission_outlier(counts[t]);
			outlier[t]=sample_multinom_one( rgen,ind,weight,3);

		}

		delete []ind;	delete []weight;
	}
	else {for (int t=0;t<(T-1);t++){ outlier[t]=0;}}
}

void level_estimation::sample_emission_param(gsl_rng * rgen,bool yes){
	if(yes){
		double proposed;
		double proba_phi=0;	double proba_proposed=0;

		for(int t=0; t<(T);t++){
			if(u[t]>0&&outlier[t]==0){
				proba_phi=proba_phi+log(MODE.emission(counts[t],s[t]*u[t],context[t],MODE.max_u,MODE.discretize_step));}}
double kappa_prop=200;
		proposed=gsl_ran_gamma(rgen,kappa_prop,MODE.phi/kappa_prop); // propose a new phi
		for(int t=0; t<(T);t++){
			if(u[t]>0&&outlier[t]==0){
				proba_proposed=proba_proposed+log(MODE.emission(counts[t],s[t]*u[t],context[t],MODE.max_u,MODE.discretize_step));}}

		double ran=log(gsl_ran_flat(rgen,0,1));
		/*apply priors: exponential of mean 2*/
		double phimean_prior=2;

		if( ran<(proba_proposed +
				log(gsl_ran_gamma_pdf(MODE.phi,kappa_prop,proposed/kappa_prop))+log(gsl_ran_exponential_pdf(proposed,phimean_prior))
				- proba_phi
				-log(gsl_ran_gamma_pdf(proposed,kappa_prop,MODE.phi/kappa_prop))-log(gsl_ran_exponential_pdf(MODE.phi,phimean_prior))))
		{param[9]=proposed;}
		else {param[9]=MODE.phi;}
	}
	else {param[9]=MODE.phi;}

}

void level_estimation::sample_error_param(gsl_rng * rgen, bool eps, bool eps2){

	//	sample frequence of the 2 errors
	double* error_param= new double[3];
	for (int i=0;i<3;i++){ error_number[i]=0;}
	for (int t=0;t<(T-1);t++){	error_number[outlier[t]]++; }

	error_number[0]=error_number[0]+100; error_number[1]=error_number[1]+1;error_number[2]=error_number[2]+1;

	if (eps&eps2){
		gsl_ran_dirichlet (rgen, 3,error_number, error_param);
		param[11]=error_param[1];
		param[12]=error_param[2];

		param[13]=MODE.ymax;
	}
	else if  (eps){
		param[11]=gsl_ran_beta (rgen, error_number[1],error_number[0]);
		param[12]=MODE.epsilon2; param[13]=MODE.ymax;}

	else if  (eps2){
		param[12]=gsl_ran_beta (rgen, error_number[2],error_number[0]);
		param[11]=MODE.epsilon; param[13]=MODE.ymax;}


	else { param[11]=MODE.epsilon;	param[12]=MODE.epsilon2; param[13]=MODE.ymax;}

    if(param[11]>0.02) param[11]=0.02; if(param[12]>0.02) param[12]=0.02;

	delete [] error_param;
}

void level_estimation::sample_particle_break_NEW(gsl_rng* rgen ){

	for(int j=0;j<MODE.npars;j++){ count_bk[j]=0;}
	double* weight= new double[3];
	int* indice=new int[3];	for (int i=0;i<3;i++){ indice[i]=(2+i);}

	for (int seg=0;seg<SEGM.number_segments;seg++){
		for (int t=SEGM.segments[seg].first;t<SEGM.segments[seg].second;t++){


			/* alfa=0, beta_0=1, beta=2, gama_u=3, gama_d=4  1- eta= 5, eta=6;*/
			if (u[t]<litle){
				if(indicator(u[t],u[t+1])==1){bk[t]=5;}
				else {bk[t]=6; }}

			else {
				if(indicator(u[t],u[t+1])==1){bk[t]=0;}
				else {
					if (u[t+1]<litle) { bk[t]= 1;}
					else {
						weight[0]= MODE.beta*MODE.shift(u[t+1]);
						weight[1]= MODE.gamma_u*indicator(0, u[t+1],u[t])* MODE.drift_up(u[t+1],u[t]);
						weight[2]= MODE.gamma_d*indicator(0,u[t],u[t+1])*MODE.drift_down(u[t+1],u[t]);
						bk[t]=sample_multinom_one( rgen,indice,weight,3);
					}
				}
			}
			count_bk[bk[t]]++;
		}
	}

	delete []indice;	delete []weight;
}

void level_estimation::sample_amplitude_param_NEW (gsl_rng * rgen, bool lambda,bool zeta,bool precision) {

	double diff_u=0, diff_d=0;
	double diff_shift=0, diff_log_shift=0;// SS_log_shift=0;

	for (int seg=0;seg<SEGM.number_segments;seg++){
		for (int t=SEGM.segments[seg].first;t<SEGM.segments[seg].second;t++){

			if (bk[t]==3) {diff_u= diff_u+(u[t+1]-u[t])/u[t];}
			if (bk[t]==4) {diff_d= diff_d+(u[t]-u[t+1])/u[t+1];}
			if (bk[t]==2||bk[t]==6||bk[t]==9) {
				diff_shift= diff_shift+u[t+1]; diff_log_shift= diff_log_shift+log(u[t+1]);

			}
		}}

	int shifts=count_bk[2]+count_bk[6];

	if(lambda){
		/* priors for  lambda param = exp(rate= 0.1) */
		param[7]= gsl_ran_gamma (rgen, (count_bk[3]+1),1/(0.1+diff_u) );
		param[8]= gsl_ran_gamma (rgen, (count_bk[4]+1),1/(0.1+diff_d) );
	}
	else {param[7]= MODE.lambda_u; param[8]=MODE.lambda_d;}



	if(zeta==1){
		double priorscale=1, priorshape=1; //mean and precision of normal prior
		param[10]=	gsl_ran_gamma(rgen, priorshape+shifts, 1/(1/priorscale+diff_shift));}

	else {	param[10]=MODE.zeta;}


}

void level_estimation::sample_param_NEW(gsl_rng * rgen,	bool alpha, bool beta_0, bool beta, bool gamma, bool eta){
	/// Sample TRANSITION parameters

	for (int i=0;i<5;i++){ count_bkt[i]=count_bk[i];	}
	/// apply priors
	count_bkt[0]= count_bkt[0]+50;	count_bkt[1]= count_bkt[1]+0.5;
	count_bkt[2]= count_bkt[2]+1.1;	count_bkt[3]= count_bkt[3]+1.5;	count_bkt[4]= count_bkt[4]+0.5;

	if(alpha&beta_0&beta&gamma){
		gsl_ran_dirichlet (rgen, 5,count_bkt, param);}


	else if(!alpha&beta_0&beta&gamma){

		count_bkt[0]=count_bkt[4];
		gsl_ran_dirichlet (rgen, 4,count_bkt, param);
		param[4]=param[0]*(1-MODE.alpha);
		param[1]=param[1]*(1-MODE.alpha);		param[2]=param[2]*(1-MODE.alpha);		param[3]=param[3]*(1-MODE.alpha);

		param[0]=MODE.alpha;
	}

	else if(alpha&	!beta_0&	beta&	!gamma){

		param[0]=gsl_ran_beta (rgen, count_bkt[0], count_bkt[2]); param[2]=1-param[0];
		param[1]=MODE.beta_0; param[3]=MODE.gamma_u; param[4]=MODE.gamma_d;}


	else if(alpha&	beta_0&		beta&	!gamma){
		gsl_ran_dirichlet (rgen, 3,count_bkt, param);
		param[3]=MODE.gamma_u;param[4]=MODE.gamma_d;}


	else if(alpha&	!beta_0&		beta&	gamma){
		count_bkt[1]=count_bkt[4];
		gsl_ran_dirichlet (rgen, 4,count_bkt, param);
		param[4]=param[1];param[1]=MODE.beta_0;}


	else if  (!alpha&	!beta_0&	!beta&	!gamma){ 	param[0]=MODE.alpha;param[1]=MODE.beta_0;param[2]=MODE.beta;	param[3]=MODE.gamma_u;param[4]=MODE.gamma_d;}

	if(eta){param[5]=gsl_ran_beta (rgen, (count_bk[5]+100), (count_bk[6]+1));	param[6]=1-param[5];}
	else {	param[6]=MODE.eta; param[5]=1-param[6];}

}

void level_estimation::sample_outlier_NEW(gsl_rng* rgen,bool yes ){
	if(yes){
		int* ind=new int[3];	for (int i=0;i<3;i++){ind[i]=i;}	double* weight= new double[3];

		for (int seg=0;seg<SEGM.number_segments;seg++){
			for (int t=SEGM.segments[seg].first;t<=SEGM.segments[seg].second;t++){

				weight[0]= (1-MODE.epsilon-MODE.epsilon2)*MODE.emission(counts[t],s[t]*u[t],context[t],MODE.max_u,MODE.discretize_step);
				weight[1]= MODE.epsilon*MODE.emission_background(counts[t],context[t]);
				weight[2]= MODE.epsilon2*MODE.emission_outlier(counts[t]);
				outlier[t]=sample_multinom_one( rgen,ind,weight,3);
			}
		}

		delete []ind;	delete []weight;
	}
	else {for (int t=0;t<(T-1);t++){ outlier[t]=0;}}
}

void level_estimation::sample_emission_param_NEW(gsl_rng * rgen,bool yes){
	param[17]=MODE.kappa;
	param[18]=MODE.a;
}

void level_estimation::sample_error_param_NEW(gsl_rng * rgen, bool eps, bool eps2){

	//	sample frequence of the 2 errors
	double* error_param= new double[3];
	for (int i=0;i<3;i++){ error_number[i]=0;}

	for (int seg=0;seg<SEGM.number_segments;seg++){
		for (int t=SEGM.segments[seg].first;t<=SEGM.segments[seg].second;t++){
			error_number[outlier[t]]++;
		}
	}

	error_number[0]=error_number[0]+100; error_number[1]=error_number[1]+1;error_number[2]=error_number[2]+1;

	if (eps&eps2){
		gsl_ran_dirichlet (rgen, 3,error_number, error_param);
		param[11]=error_param[1];
		param[12]=error_param[2];

		param[13]=MODE.ymax;
	}
	else if  (eps){
		param[11]=gsl_ran_beta (rgen, error_number[1],error_number[0]);
		param[12]=MODE.epsilon2; param[13]=MODE.ymax;}

	else if  (eps2){
		param[12]=gsl_ran_beta (rgen, error_number[2],error_number[0]);
		param[11]=MODE.epsilon; param[13]=MODE.ymax;}


	else { param[11]=MODE.epsilon;	param[12]=MODE.epsilon2; param[13]=MODE.ymax;}

    if(param[11]>0.02) param[11]=0.02; if(param[12]>0.02) param[12]=0.02;

	delete [] error_param;
}

void level_estimation::average_sample_param_protocol (const char* file){

	int iter=0; double temp;
	ifstream fileparam;
	fileparam.open(file);
	for (int i=0;i<MODE.npars; i++){param[i]=MODE.get_param()[i];}
	for (int i=15;i<MODE.npars; i++){param[i]=0;}
	if (!fileparam) {	cerr << "\n You didn' sample the parameters \n" << endl;}

	fileparam>>temp;
	while(fileparam){
        	iter++;
       		for (int i=0;i<MODE.npars; i++){            if(i>=15)   {param[i]+=temp;}            fileparam>>temp;            }
	}


	cout<<"local variability and read count emission parameters averaged from: "<<iter<<"iterations. From file; "<<file<<endl;

	for (int i=15;i<MODE.npars; i++){ param[i]=param[i]/iter; cout<<param[i]<<" ";}
	cout<<endl;
}

void level_estimation::average_sample_param (const char* file){

    cout<<"Use average parameters estimated in previous iterations: "<<endl;
	int iter=0; double temp;
	ifstream fileparam;
	fileparam.open(file);
	for (int i=0;i<MODE.npars; i++){param[i]=0;}

	if (!fileparam) {	cerr << "\n You are missing the sampled parameters \n" << endl;}

	fileparam>>temp;
	while(fileparam){
        	iter++;
       		for (int i=0;i<MODE.npars; i++){           param[i]+=temp;         fileparam>>temp;            }
	}

	cout<<"Parameters averaged from "<<iter<<" iterations from the file "<<file<<endl<<endl;
	for (int i=0;i<MODE.npars; i++){ param[i]=param[i]/iter; cout<<param[i]<<" ";}
	cout<<endl<<endl;
}

void level_estimation::get_proposal_weights(double * weights, bool* duplicated,int **ywrange, double**swrange){


// here for each component $d$ and each $t$ we compute $weights$  the sum of the component/ full proposal
    double *temp;
    temp=new double[PROP.window+4];
    for (int d=0;d<(PROP.window+4);d++)
        temp[d]=0;

    for (int d=0;d<(PROP.window+3);d++)
        weights[d]=0;
    double sumweights=0;

    for( int t=1;t<T;t++ ){
        PROP.set_u(u[t-1]);
        PROP.set_y_range(ywrange[t],swrange[t],duplicated[t]);
        if(PROP.duplicated||(PROP.ut<litle)) continue;
        PROP.proba_components(temp,u[t]);
//        if(abs(PROP.ut-u[t])>litle){cout<<"w0: "<<PROP.ut<<" "<<u[t]<<" "<<temp[0]<<" "<<temp[1]<<" "<<temp[2]<<" "<<temp[3]<<" "<<temp[4]<<" "<<temp[PROP.window+3]<<endl;usleep(100000);}
        for (int d=0;d<(PROP.window+3);d++)
            weights[d]=weights[d]+temp[d]/temp[PROP.window+3];
    }

    for (int d=0;d<(PROP.window+3);d++)
          sumweights=sumweights+weights[d];
    for (int d=0;d<(PROP.window+3);d++)
        PROP.wmixture[d]=weights[d]/sumweights;
    PROP.wmixture_cum[0]=PROP.wmixture[0];
    for (int d=1;d<(PROP.window+3);d++)
        PROP.wmixture_cum[d]=PROP.wmixture_cum[d-1]+PROP.wmixture[d];
    delete temp;
    cout<<PROP.wmixture_cum[0]<<endl;
}




vector<float>* calc_emission_table_log_old ( string counts_folder,float kappa,float theta, int max_u,float divide_u){

    unsigned int intervalsu=(unsigned int)floor(log(max_u)*divide_u+138.5+1);
    cout<<"discretize the log expresson level in: "<<intervalsu<<endl;
    size_t nmin,nmax;
    vector<float>* proba=new vector<float> [intervalsu+1];

    float *u=new float[intervalsu+1];
    float *lu=new float[intervalsu+1];

    for(unsigned int i=0;i<=intervalsu;i++){ lu[i]=log(0.001)+i/divide_u;}
    for(unsigned int i=0;i<=intervalsu;i++){ u[i]=exp(lu[i]);}
    int countmax;
    float temp5;
    stringstream tempstr; tempstr<<kappa;
    stringstream tempstr2; tempstr2<<theta;
    stringstream tempstr3; tempstr3<<max_u;
    stringstream tempstr4; tempstr4<<divide_u;
    string emissionfile=string(counts_folder+"emission_table_log_"+tempstr3.str()+"_"+tempstr4.str()+"_"+tempstr.str()+"_"+tempstr2.str());

    /*************************************************/
    ifstream fileemisin;
    fileemisin.open(emissionfile.c_str());

    if(fileemisin.good()) {
        cout<<"Reading emission function from the file:"<<emissionfile<<endl;

    for (unsigned int g=0;g<=intervalsu;g++){
        fileemisin>>u[g]; fileemisin>>countmax;  proba[g].push_back(u[g]); proba[g].push_back(countmax);
        for (int i=0;i<=countmax;i++){
             fileemisin>>temp5;
             proba[g].push_back(temp5);
        }
    }
        fileemisin.close();
    }
    else {
        fileemisin.close();

    cout<<"Emission table "<<emissionfile<<" doesn't exists and must be created."<<endl;
    cout<<"Started computing emission function"<<endl;

    double limit=pow(10.0,-6.0);
    countmax=50; //int temp;
    float probatemp;
////////////    for mean =0
    proba[0].push_back(u[0]);
    proba[0].push_back(countmax);
    proba[0].push_back(1);
    for ( int i=1;i<=countmax;i++){ proba[0].push_back(0);}

    for( unsigned int g=1;g<=intervalsu;g++){
        nmin=max((int)(u[g]/(theta*kappa)/1.750)-100,1);
        nmax=(int)(u[g]/(theta*kappa)*1.5+100);
        //// compute the probability at count temp. if it is less than the limit then increse temp with a position
        do {
            probatemp=0;
            for (unsigned int n=nmin;n<=nmax;n++){
                probatemp+=gsl_ran_poisson_pdf(n,u[g]/(theta*kappa))*gsl_ran_negative_binomial_pdf(countmax,1-n*theta/(n*theta+1),kappa);
            }
            probatemp>limit?countmax++:1;
        }   while(probatemp>limit);

        proba[g].push_back(u[g]);
        proba[g].push_back(countmax);
        for ( int i=0;i<=countmax;i++){
            probatemp=0;
            if(i==0){  probatemp=gsl_ran_poisson_pdf(0,u[g]/(theta*kappa));}
            for (unsigned int n=nmin;n<=nmax;n++){
                probatemp+=gsl_ran_poisson_pdf(n,u[g]/(theta*kappa))*gsl_ran_negative_binomial_pdf(i,1-n*theta/(n*theta+1),kappa);
           }
            proba[g].push_back(probatemp);
        }
              if(g%10==0)  cout<<u[g]<<" "<<u[g]/(theta*kappa)<<" "<< (proba[g])[1]<<endl;
   }

    cout<<"Ended computing emission function"<<endl;

    ofstream fileemis;
    fileemis.open((emissionfile).c_str());

    for( unsigned int g=0; g<=intervalsu; g++){

            for ( unsigned int i=0;i<proba[g].size();i++){
                fileemis<<proba[g][i]<<"\t";
            }
            fileemis<<endl;
    }
    fileemis.close();
    }

    delete u;

    return proba;
}

vector<float>* calc_emission_table_log ( string counts_folder,float kappa,float a, int max_u,float divide_u){

    unsigned int intervalsu=(unsigned int)floor(log(max_u)*divide_u+138.5+1);
    cout<<"discretize the log expresson level in: "<<intervalsu<<endl;
    size_t nmin,nmax;
    vector<float>* proba=new vector<float> [intervalsu+1];

    float *u=new float[intervalsu+1];
    float *lu=new float[intervalsu+1];

    for(unsigned int i=0;i<=intervalsu;i++){ lu[i]=log(0.001)+i/divide_u;}
    for(unsigned int i=0;i<=intervalsu;i++){ u[i]=exp(lu[i]);}
    int countmax;
    float temp5;
    stringstream tempstr; tempstr<<kappa;
    stringstream tempstr2; tempstr2<<a;
    stringstream tempstr3; tempstr3<<max_u;
    stringstream tempstr4; tempstr4<<divide_u;
    string emissionfile=string(counts_folder+"emission_table_log_"+tempstr3.str()+"_"+tempstr4.str()+"_"+tempstr.str()+"_"+tempstr2.str());

    /*************************************************/
    ifstream fileemisin;
    fileemisin.open(emissionfile.c_str());

    if(fileemisin.good()) {
        cout<<"Reading emission function from the file:"<<emissionfile<<endl;

    for (unsigned int g=0;g<=intervalsu;g++){
        fileemisin>>u[g]; fileemisin>>countmax;  proba[g].push_back(u[g]); proba[g].push_back(countmax);
        for (int i=0;i<=countmax;i++){
             fileemisin>>temp5;
             proba[g].push_back(temp5);
        }
    }
        fileemisin.close();
    }
    else {
        fileemisin.close();

    cout<<"Emission table "<<emissionfile<<" doesn't exists and must be created."<<endl;
    cout<<"Started computing emission function"<<endl;

    double limit=pow(10.0,-7.0);
    countmax=50; //int temp;
    float probatemp;
////////////    for mean =0
    proba[0].push_back(u[0]);
    proba[0].push_back(countmax);
    proba[0].push_back(1);
    for ( int i=1;i<=countmax;i++){ proba[0].push_back(0);}

    for( unsigned int g=1;g<=intervalsu;g++){
        nmin=max((int)(u[g]/a/1.750)-100,1);
        nmax=(int)(u[g]/a*1.5+100);
        //// compute the probability at count temp. if it is less than the limit then increse temp with a position
        do {
            probatemp=0;
            for (unsigned int n=nmin;n<=nmax;n++){
                probatemp+=gsl_ran_poisson_pdf(n,u[g]/a)*gsl_ran_negative_binomial_pdf(countmax,1-n*a*kappa/(n*a*kappa+1),1/kappa);
            }
            probatemp>limit?countmax++:1;
        }   while(probatemp>limit);

        proba[g].push_back(u[g]);
        proba[g].push_back(countmax);
        for ( int i=0;i<=countmax;i++){
            probatemp=0;
            if(i==0){  probatemp=gsl_ran_poisson_pdf(0,u[g]/a);}
            for (unsigned int n=nmin;n<=nmax;n++){
                probatemp+=gsl_ran_poisson_pdf(n,u[g]/a)*gsl_ran_negative_binomial_pdf(i,1-n*a*kappa/(n*a*kappa+1),1/kappa);
           }
            proba[g].push_back(probatemp);
        }
              if(g%10==0)  cout<<u[g]<<" "<<u[g]/a<<" "<< (proba[g])[1]<<endl;
   }

    cout<<"Ended computing emission function"<<endl;

    ofstream fileemis;
    fileemis.open((emissionfile).c_str());

    for( unsigned int g=0; g<=intervalsu; g++){

            for ( unsigned int i=0;i<proba[g].size();i++){
                fileemis<<proba[g][i]<<"\t";
            }
            fileemis<<endl;
    }
    fileemis.close();
    }

    delete u;

    return proba;
}

vector<float>* calc_emission_table_NB ( string counts_folder,float kappa,float theta,float ks, size_t intervalsu,float divide_u){

    size_t nmin,nmax;
    vector<float>* proba=new vector<float> [intervalsu+1];

    float *u=new float[intervalsu+1];
    for(unsigned int i=0;i<=intervalsu;i++){ u[i]=(float)i/divide_u;}

    int countmax;
//    float countmaxf;
    float temp5;
    stringstream tempstr; tempstr<<kappa;
    stringstream tempstr2; tempstr2<<theta;
    stringstream tempstr22; tempstr22<<ks;
    stringstream tempstr3; tempstr3<<intervalsu;
    stringstream tempstr4; tempstr4<<divide_u;
    string emissionfile=string(counts_folder+"emission_table_nb_"+tempstr3.str()+"_"+tempstr4.str()+"_"+tempstr.str()+"_"+tempstr2.str()+"_"+tempstr22.str());

    /*************************************************/
    ifstream fileemisin;
    fileemisin.open(emissionfile.c_str());

    if(fileemisin.good()) {
        cout<<"Reading emission function fom the file:"<<emissionfile<<endl;

    for (unsigned int g=0;g<intervalsu;g++){
        fileemisin>>u[g]; fileemisin>>countmax;  proba[g].push_back(countmax);
        for (int i=0;i<=countmax;i++){
             fileemisin>>temp5;
             proba[g].push_back(temp5);
        }
    }
        fileemisin.close();
    }
    else {
        fileemisin.close();

    cout<<"Emission table "<<emissionfile<<" doesn't exists and must be created."<<endl;
    cout<<"Started computing emission function"<<endl;
//cout<<"write nb:"<< gsl_ran_negative_binomial_pdf(1,ks/(ks+1/(3*0.5)),ks)<<endl;
    double limit=pow(10.0,-6.0);
    countmax=50; //int temp;
    float probatemp;
////////////    for mean =0
    proba[0].push_back(countmax);
    proba[0].push_back(1);
    for ( int i=1;i<=countmax;i++){ proba[0].push_back(0);}

    for(unsigned int g=1;g<=intervalsu;g++){

        nmin=max((int)(u[g]/1.50)-100,1);
        nmax=(int)(u[g]*1.25+100);

        //// compute the probability at count temp. if it is less than the limit then increse temp with a position

        do {
            probatemp=0;
            for (unsigned int n=nmin;n<=nmax;n++){

                probatemp+=gsl_ran_negative_binomial_pdf(n,ks/(ks+u[g]),ks)*gsl_ran_negative_binomial_pdf(countmax,1-n*theta/(n*theta+1),kappa);
            }
            probatemp>limit?countmax++:1;
        }   while(probatemp>limit);

        proba[g].push_back(countmax);
        for ( int i=0;i<=countmax;i++){
            probatemp=0;
            if(i==0){  probatemp=gsl_ran_negative_binomial_pdf(0,ks/(ks+u[g]),ks);}


            for (unsigned int n=nmin;n<=nmax;n++){
                probatemp+=gsl_ran_negative_binomial_pdf(n,ks/(ks+u[g]),ks)*gsl_ran_negative_binomial_pdf(i,1-n*theta/(n*theta+1),kappa);
           }

            proba[g].push_back(probatemp);
        }
   }

    cout<<"Ended computing emission function"<<endl;

    ofstream fileemis;
    fileemis.open((emissionfile).c_str());

    for( unsigned int g=0; g<=intervalsu; g++){
        fileemis<<u[g]<<"\t";
            for ( unsigned int i=0;i<proba[g].size();i++){
                fileemis<<proba[g][i]<<"\t";
            }
            fileemis<<endl;
    }
    fileemis.close();
    }

    delete u;

    return proba;
}
