#ifndef DEF_MODEL_H
#define DEF_MODEL_H


#include "functions.h"
#include "data.h"

class model{

public:
    double alpha, beta;
	double zeta;
	double phi;
	int npars;

	model(){}
	~model(){}
	model( double xalfa, double xbeta, double xc, double xphi, int xnpars);

	friend class level_estimation;
};


class model2: public model{

    public:

    double gamma_u, gamma_d, lambda_u, lambda_d, eta, beta_0;
	double epsilon, epsilon2,ephi, ymax, ymean,yphi;// 2 typers of errorrs background nois and transcript error

    double* param;
	double alpha_struct, struct_shape;
    double  kappa, a;
    vector<float>* nb_table;
    int max_u;float discretize_step;
	model2(){}
	model2( double xalfa, double xbeta, double xbeta_0,
			double xgamma_u, double xgamma_d, double xeta, double xlambda_u, double xlambda_d,
			double xc,double kappa, double a, double xphi, double xalpha_struct, double xstruct_shape,
			double xepsilon, double xepsilon2, double xephi, double xymax, double xemean,
			int xnpars);

	model2( double* param, double ymax, double ymean,double yphi, int npars, vector<float>* nb_table, int max_u, float discretize_step);
//    ~model2(){ delete param; delete nb_table_p;}
	double shift_0 (double ut1);
	double shift (double ut1);


	double drift_up (double ut1,double ut);
	double drift_down (double ut1,double ut);
	//	double kernel_hidden (double ut1,double ut, int mt);

	double kernel_hidden (double ut1, double ut);
	double sample_kernel (gsl_rng* rgen, double ut);

	double kernel_hidden_0 (double ut1);

	double shift_struct(double st1);
	double kernel_struct(double st1,double st);
	double kernel_struct_0(double st);

	double emission_background (int y,double context);
	double emission_outlier (int y);

	double emission (int y, double mean, double context,int max_u,float discretize_step);
	double emission_proba_struct_duplicated (int y, double mean,double structure, double context, int outlier,int duplicated);

	void update_param(double* param);
	void update_param_struct(double* param);
	void update_ymean_ymax(double ymean,double ymax);
	void update_nb_table(vector<float>* nb_table);
	double* get_param();

	friend class level_estimation;
	friend class evaluation;
	friend class data_simulation;
    friend class proposal;
};



class proposal{

public:

	double eta_p, alpha_p, alpha_p_struct, at, bt , ct,  epsilon_p, zetap; // proposal parameters

	double ut, ywmt,st, duplicated, swmt;
	double*  wmixture,  *wmixture_cum;
	int* ywmt_range; double*swmt_range;

    model2 * MODE;
	int mt;
	int window; // the size of the first window of counts
	int *window_range;
	proposal(){}
	proposal( double xut, double xst, int xywmt,	 bool dupwt, double xalpha_p_struct,  double  xzetap, int window);
	double sample(gsl_rng* rgen);
    float sample_NEW(gsl_rng* rgen);
    float proba_NEW(float u);
	double proba(double u);
	void proba_components(double* probas,double u);/// component d of the proposal (which is a mixture ow window+2 functions)

	double sample_poz0(gsl_rng* rgen);
	double proba_poz0(double u);

	double sample_level(gsl_rng* rgen);
	double proba_level(double u);
	double sample_level_poz0(gsl_rng* rgen);
	double proba_level_poz0(double u);

	void set_u(double u){ut=u;}
	void set_s(double s){st=s;}
	void set_model(model2 MODE1){*MODE=MODE1;}
	void set_alphap(double alphap){this->alpha_p=alphap;}
//    void update_weights(double* weights);
    double* get_weights(double* weights){ return wmixture;};
	void set_zetap(double zetap){this->zetap=zetap;}
	void set_alphap_struct(double alpha_p_struct){this->alpha_p_struct=alpha_p_struct;}
	void set_y(float yk, double sk, int dup){ywmt=yk;swmt=sk;duplicated=dup;}
    void set_y_range(int* yk, double* sk, int dup){ywmt_range=yk;swmt_range=sk;duplicated=dup;}
	int get_mt(){return mt;}

	double	proba_struct(double s);
	double	sample_struct(gsl_rng* rgen);
	double	proba_struct_0(double s);
	double	sample_struct_0(gsl_rng* rgen);

	~proposal(){}
};
//





class data_segments{

public:
	vector <pair<int,int> > segments; // contains start and end position (0 included)
//	int* ind;  // 0 or 1.. 1 says it is included in param estim
	int T;	int T0;
	string strand;
	string crom;
	vector <pair<int,int> >  positions_for_param; //contains segment number and position on chr
	int length_segments;
	int number_segments;
	data_segments(){};
	data_segments(string count_folder,string parseq_folder,bool highexpressed_present, bool local_variability, string file_segments, int T, int T0,string strand,string crom );
	data_segments( string results_folder,int T,int* counts,string strand,string crom);


	~data_segments(){};
	void sample(gsl_rng * rgen, int* pozs, double proba_0);

};

//class data_segments;
class level_estimation{

public:
	double* u; double* s;
	double* context;
	int* bk, * bks;
	int* outlier;
	int* count_bk; double* count_bkt;
	int T;
	int* counts;
	double* param;
	model2 MODE;
    proposal PROP;

	data_segments SEGM; // indicator that shows which positions are taken into account for parameter estimation
	double* error_number;
	double* prop_alpha_s,* w_alpha_s;


public:
	level_estimation(int T, int* counts, model2 mode, double* context, data_segments segm);
    level_estimation(int T, int* counts, model2 mode, proposal prop, double* context);
	~ level_estimation();
	void update_estimation (double* u,double* s, model2 mode1 );
	void update_estimation_counts (int* y);
	void update_estimation_context (double* context);
	void sample_particle_break (gsl_rng* rgen);
	void sample_param (gsl_rng* rgen, bool alpha, bool beta_0, bool beta, bool gamma, bool gamma_2,bool lambda,bool zeta,bool precision,bool eta);
	void sample_outlier(gsl_rng * rgen,bool yes);

	void sample_emission_param(gsl_rng * rgen,bool yes);
	void sample_error_param(gsl_rng * rgen,bool eps, bool eps2);
	void sample_struct_param(gsl_rng * rgen,bool alpha_s,bool shape_s);

	void sample_particle_break_NEW (gsl_rng* rgen);
	void sample_param_NEW (gsl_rng* rgen, bool alpha, bool beta_0, bool beta, bool gamma, bool eta);
	void sample_amplitude_param_NEW (gsl_rng* rgen,bool lambda,bool zeta,bool precision);
	void sample_outlier_NEW(gsl_rng * rgen,bool yes);

	void sample_emission_param_NEW(gsl_rng * rgen,bool yes);
	void sample_error_param_NEW(gsl_rng * rgen,bool eps, bool eps2);
	void sample_struct_param_NEW(gsl_rng * rgen,bool alpha_s,bool shape_s);

	void average_sample_param_protocol (const char* file);
	void average_sample_param (const char* file);

    void get_proposal_weights(double * weights, bool* duplicated,int **ywrange, double **swrange);


};

void determine_high_segments( string results_folder, int T,int* counts,string strand,string crom);
void segment_sw( string results_folder, int T,int* counts,string strand,string crom);

//vector<float>* calc_emission_table (string counts_folder, float kappa,float theta,size_t intervalsu,float divide_u);
vector<float>* calc_emission_table_NB ( string counts_folder,float kappa,float theta,float ks, size_t intervalsu,float divide_u);
vector<float>* calc_emission_table_log_old ( string counts_folder,float kappa,float theta, int max_u,float divide_u);
void estimate_emission_parameters( gsl_rng * rgen, string results_folder,int T,int* counts,string strand,string crom, model2* MODE);
vector<float>* calc_emission_table_log ( string counts_folder,float kappa,float a, int max_u,float divide_u);
#endif
